#!/bin/sh

# chroot helper

if [ $# -ne 2 ]; then
	echo "Usage: $0 /path/to/bin /path/to/chroot"
	exit 1
fi

PROG=$1
CHROOT=$2

if [ ! -d ${CHROOT} ]; then
	echo "${CHROOT}: not a directory"
	exit 1
fi

if [ ! -f ${PROG} ]; then
	echo "${PROG}: not a file"
	exit 1
fi

if [ ! -x ${PROG} ]; then
	echo "${PROG}: not executable"
	exit 1
fi

touch ${CHROOT}/.remove_me 2>/dev/null

if [ $? -gt 0 ]; then
	echo "Can't create file into ${CHROOT}"
	exit 1
fi

/bin/rm -f ${CHROOT}/.remove_me

OPSYS=$(uname)

case "${OPSYS}" in
	OpenBSD)
		FILES=$(ldd /home/dsx/src/mohawk/mohawk | awk '$7 ~ /^\// { print $7 }')
		;;
	FreeBSD)
		FILES=$(ldd -a -f '%p\n' ${PROG} | grep -v ':$' | sort -u)
		FILES="/libexec/ld-elf.so.1
${FILES}
${PROG}
"
		;;
	NetBSD)
		FILES=$(ldd -f '%p\n' ${PROG})
		;;
	*) echo "Sorry, '${OPSYS}' is not yet supported" && exit 1
esac

tar czf - ${FILES} | tar xzf - -C ${CHROOT}/
# vim:set sw=4 sts=4 ts=4 :
