#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <err.h>

#include "mohawk.h"
#include "dump.h"

void dump_ai(struct vhost *);
const char* undef_on_off(int);
void dump_vhost(struct vhost *);

void
dump_ai(struct vhost *vh)
{
	struct address_info *ai;
	char hostname[NI_MAXHOST];
	char service[NI_MAXSERV];
	int error;

	printf("\tlisten: ");
	SLIST_FOREACH(ai, &vh->address_infos, entry) {
		if ((error = getnameinfo(ai->ai->ai_addr, ai->ai->ai_addrlen,
			                     hostname, sizeof(hostname),
			                     service, sizeof(service),
			                     NI_NUMERICHOST | NI_NUMERICSERV)) != 0) {
			warnx("getnameinfo(): %s", gai_strerror(error));
		}
		printf("['%s' '%s'] ", hostname, service);
	}
	printf("\n");
}

#define DUMP_HASH(FIELD) \
	printf("\t%s: ", #FIELD); \
	TAILQ_FOREACH(hash, &vh->FIELD, entry) { \
		printf("['%s' => '%s'] ", hash->key, hash->value); \
	} \
	printf("\n");

#define DUMP_LIST(FIELD) \
	printf("\t%s: ", #FIELD); \
	TAILQ_FOREACH(list, &vh->FIELD, entry) { \
		printf("'%s' ", list->item); \
	} \
	printf("\n");

const char *
undef_on_off(int value)
{
	if (value == -1)
		return "undef";
	if (value)
		return "on";
	return "off";
}

void
dump_vhost(struct vhost *vh)
{
	struct mohawk_list *list;
	struct mohawk_hash *hash;

	printf("\nVirtual host '%s'\n", vh->host);
	printf("\trootdir: '%s'\n", vh->rootdir);
	printf("\tauth_path: '%s'\n", vh->auth_path ? vh->auth_path : "");
	printf("\tcharset: '%s'\n", vh->charset ? vh->charset : "");
	printf("\tdirlist_css_url: '%s'\n", vh->dirlist_css_url ? vh->dirlist_css_url : "");
	printf("\tmohawk_name: '%s'\n", vh->mohawk_name);
	printf("\tstatus_url: '%s'\n", vh->status_url ? vh->status_url : "");
	printf("\tauthentication: '%s'\n", undef_on_off(vh->authentication));
	printf("\tcgi_expose_mohawk_version: '%s'\n", undef_on_off(vh->cgi_expose_mohawk_version));
	printf("\tdirlist: '%s'\n", undef_on_off(vh->dirlist));
	printf("\tmaxage: '%d'\n", vh->maxage);
	printf("\tx_forwarded_for: '%s'\n", undef_on_off(vh->x_forwarded_for)); 
	printf("\temail_admin: '%s'\n", vh->email_admin ? vh->email_admin : "");

	DUMP_LIST(cgi_patterns);
	DUMP_LIST(index_names);
	DUMP_LIST(no_cgi_maps);
	DUMP_LIST(no_log_patterns);
	DUMP_LIST(restrict_froms);

	DUMP_HASH(cgi_env);
	DUMP_HASH(cgi_map);
	printf("\tcgi_env_count: '%d'\n", vh->cgi_env_count);

	dump_ai(vh);
	return;
}

void
dump_config(void)
{
	struct vhost *vh;
	struct mohawk_hash *hash;

	printf("Dump mohawk config:\n\n");
	printf("debug: '%s'\n", undef_on_off(conf.debug));
	printf("chroot: '%s'\n", conf.chroot ? conf.chroot : "");
	printf("pidfile: '%s'\n", conf.pidfile ? conf.pidfile : "");
	printf("syslog facility: '%s'\n", conf.syslog_facility);
	printf("user: '%s'\n", conf.user);

	printf("mime_type: ");
	TAILQ_FOREACH(hash, &conf.mime_type, entry) {
		printf("['%s' => '%s'] ", hash->key, hash->value);
	}
	printf("\n");

	dump_vhost(conf.default_vhost);
	SLIST_FOREACH(vh, &conf.vhosts, entry)
		dump_vhost(vh);

	return;
}

/* vim: set sw=4 sts=4 ts=4 : */
