/* mohawk - small HTTP server
**
** Copyright © 2010 by Baptiste Daroussin <bapt@FreeBSD.org>,
** Copyright © 2010 by Freddy Dissaux <freddy.dsx@free.fr>,
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
*/

#ifndef _MOHAWK_H
#define _MOHAWK_H

#include <sys/queue.h>
#include <sys/types.h>
#if defined(__FreeBSD__)
#include <libutil.h>
#endif
#if defined(__NetBSD__)
#include <util.h>
#endif

#include <netinet/in.h>

#if defined(__OpenBSD__)
#include "compat.h"
#endif

#define MOHAWK_VERSION "2.0"

#ifndef DEFAULT_AUTH_FILE
#define DEFAULT_AUTH_FILE ".htpasswd"
#endif /* DEFAULT_AUTH_FILE */

#ifndef DEFAULT_HTTP_PORT
#define DEFAULT_HTTP_PORT "80"
#endif /* DEFAULT_HTTP_PORT */

#ifndef DEFAULT_PIDFILE
#define DEFAULT_PIDFILE "/var/run/mohawk.pid"
#endif /* DEFAULT_PIDFILE */

#ifndef DEFAULT_SYSLOG_FACILITY
#define DEFAULT_SYSLOG_FACILITY "daemon"
#endif /* DEFAULT_SYSLOG_FACILITY */

#ifndef DEFAULT_USER
#define DEFAULT_USER "nobody"
#endif /* DEFAULT_USER */

#ifdef __NetBSD__
#define IPPORT_MAX IPPORT_ANONMAX
#endif

#ifdef __OpenBSD__
#define IPPORT_MAX IPPORT_HILASTAUTO
#endif

struct mohawk_list {
	char *item;
	TAILQ_ENTRY(mohawk_list) entry;
};

struct mohawk_hash {
	char *key;
	char *value;
	TAILQ_ENTRY(mohawk_hash) entry;
};

struct address_info {
	struct addrinfo *ai;
	SLIST_ENTRY(address_info) entry;
};

struct vhost {
	TAILQ_HEAD(, mohawk_list) auth_patterns;
	TAILQ_HEAD(, mohawk_list) cgi_patterns;
	TAILQ_HEAD(, mohawk_list) index_names;
	TAILQ_HEAD(, mohawk_list) no_auth_patterns;
	TAILQ_HEAD(, mohawk_list) no_cgi_maps;
	TAILQ_HEAD(, mohawk_list) no_log_patterns;
	TAILQ_HEAD(, mohawk_list) restrict_froms;
	TAILQ_HEAD(, mohawk_hash) cgi_env;
	TAILQ_HEAD(, mohawk_hash) cgi_map;
	SLIST_HEAD(, address_info) address_infos;
	const char *email_admin;
	const char *auth_path;
	const char *charset;
	const char *dirlist_css_url;
	const char *host;
	const char *mohawk_name;
	const char *rootdir;
	const char *status_url;
	int authentication;
	int cgi_env_count;
	int cgi_expose_mohawk_version;
	int dirlist;
	int maxage;
	int x_forwarded_for;
	SLIST_ENTRY(vhost) entry;
};

struct callback_data {
    struct vhost *vh;
    struct evhttp *httpd;
    char *host;
    char *service;
    int port;
    TAILQ_ENTRY(callback_data) entry;
};

struct mohawk_conf {
	TAILQ_HEAD(, mohawk_hash) mime_type; 
	SLIST_HEAD(, vhost) vhosts;
	TAILQ_HEAD(, callback_data) cb_datas;
	struct vhost *default_vhost;
	struct pidfh *pfh;
	char *chroot;
	const char *pidfile;
	const char *user;
	const char *syslog_facility;
	int debug;
};

extern struct mohawk_conf conf;

void parse_config(const char *);
int host(const char *host, const char *service);
int parse_number(const char *, int, int);
const char *facility_lookup(const char *, int *);

#endif
/* vim: set sw=4 sts=4 ts=4 : */
