/* mohawk - small HTTP server
**
** Copyright © 2010 by Baptiste Daroussin <bapt@FreeBSD.org>,
** Copyright © 2010 by Freddy Dissaux <freddy.dsx@free.fr>,
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
*/
%option nounput
%option noinput
%option yylineno

%{
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <err.h>
#include <errno.h>
#if defined(__FreeBSD__) || defined(__OpenBSD__)
#include <sys/limits.h>
#endif
#ifdef __NetBSD__
#include <machine/limits.h>
#endif
#if defined(__OpenBSD__)
#include "y.tab.h"
#else
#include "parse.h"
#endif

char* words(char *);
int parse_number(const char *, int, int);
int yylex(void);

char *
words(char *s)
{
	char *p;
	p = strrchr(s, '"');
	*p = '\0';
	p = s;
	p++;
	return p;
}

int
parse_number(const char *buf, int min, int max)
{
	char *ep;
	int ival;
	long lval;

	errno = 0;
	lval = strtol(buf, &ep, 10);
	if (buf[0] == '\0' || *ep != '\0')
		errx(1, "parse_number '%s': '%s'\n", buf, ep);
	if ((errno == ERANGE && (lval == LONG_MAX || lval == LONG_MIN)) || (lval > INT_MAX || lval < INT_MIN))
		errx(1, "parse_number '%s': out of range\n", buf);
	ival = lval;
	if (ival < min || ival > max)
		errx(1, "parse_number '%s': %d out of range %d %d\n", buf, ival, min, max);
	return ival;
}
%}

%%
auth_path           return AUTH_PATH;
auth_patterns       return AUTH_PATTERNS;
authentication      return AUTHENTICATION;
cgi_env             return CGI_ENV;
cgi_map             return CGI_MAP;
cgi_patterns        return CGI_PATTERNS;
charset             return CHARSET;
chroot              return CHROOT;
clone               return CLONE;
debug               return MOHAWK_DEBUG;
default             return DEFAULT;
dirlist             return DIRLIST;
dirlist_css_url     return DIRLIST_CSS_URL;
email_admin         return EMAIL_ADMIN;
index_names         return INDEX_NAMES;
listen              return LISTEN;
maxage              return MAXAGE;
mime_type           return MIME_TYPE;
mohawk_name         return MOHAWK_NAME;
no_auth_patterns    return NO_AUTH_PATTERNS;
no_cgi_maps         return NO_CGI_MAPS;
no_log_patterns     return NO_LOG_PATTERNS;
pidfile             return PIDFILE;
port                return PORT;
reset               return RESET;
restrict_froms      return RESTRICT_FROMS;
rootdir             return ROOTDIR; 
status_url          return STATUS_URL;
syslog_facility     return SYSLOG_FACILITY;
user                return USER;
vhost               return VHOST;
x_forwarded_for     return X_FORWARDED_FOR;
cgi_expose_mohawk_version return CGI_EXPOSE_MOHAWK_VERSION;
\{                  return OBRACE;
\}                  return EBRACE;
on|off              yylval.number =! strcmp(yytext, "on"); return STATE;
\"[^\"]*\"          yylval.string = strdup(words(yytext)); return WORDS;
[0-9]+              yylval.number = parse_number(yytext, 0, INT_MAX); return NUMBER;
[[:graph:]]+        yylval.string = strdup(yytext); return WORD;
[ \t]+              ;
#.*$                yylineno--;
\n                  ;
%%

/* vim: set sw=4 sts=4 ts=4 : */
